/*
 *  This sketch sends data via HTTP GET requests to data.sparkfun.com service.
 *
 *  You need to get streamId and privateKey at data.sparkfun.com and paste them
 *  below. Or just customize this script to talk to other HTTP servers.
 *
 */

#include <ESP8266WiFi.h>
#include <onewire.h>
#include <dht.h>


#define DHTPIN 5        // what digital pin we're connected to
#define DHTTYPE DHT22   // DHT 22  (AM2302), AM2321

DHT dht(DHTPIN, DHTTYPE);

const char* ssid     = "xxxx";
const char* password = "xxxxxx";


const char* host = "192.168.8.1";
const int   httpPort = 8090;
const char* streamId   = "....................";
const char* privateKey = "....................";

IPAddress Ip(192,168,8,81);
IPAddress Gw(192,168,8,1);
IPAddress Mask(255,255,255,0);
IPAddress Dns(192,168,8,1);

void WifiConnect()
{
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
 
  
  WiFi.config(Ip,Gw,Mask,Dns);
  
  if(String(ssid) != WiFi.SSID()) 
    WiFi.begin(ssid, password);
    
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
    }
  
  Serial.println("");
  Serial.println("WiFi connected");  
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}



void setup() 
{
  pinMode(BUILTIN_LED, OUTPUT);                                                   
  Serial.begin(115200);
  Serial.setDebugOutput(true);
  delay(10);

  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  // für statische Ip 
  // WiFi.config(Ip,Gw,Mask,Dns);
  
  if(String(ssid) != WiFi.SSID()) 
    WiFi.begin(ssid, password);
  
  dht.begin();
  digitalWrite(BUILTIN_LED, HIGH);                                                
}

void WriteToServer(String &rS) 
{
  // Warten bis dei Wifi Verbindung steht 
  while (WiFi.status() != WL_CONNECTED) 
    {
    delay(500);
    Serial.print(".");
    }
  
  Serial.println("");
  Serial.println("WiFi connected");  
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());

  Serial.print("connecting to ");
  Serial.println(host);
  
  // Use WiFiClient class to create TCP connections
  WiFiClient client;
  client.setNoDelay(true);
  if (!client.connect(host, httpPort)) 
    {
    Serial.println("connection failed");
    return;
    }
  
  // We now create a URI for the request
  String url = "/test.js";

  Serial.print("Requesting URL: ");
  Serial.println(url);
  
  String S=String("POST ") + url + " HTTP/1.1\r\n";
  S+=String("Host: ") + host + "\r\n" 
                               "Content-Type: text/plain\r\n" 
                               "Content-Length: ";
  S+=rS.length();
  S+="\r\n\r\n";
  S+=rS;
  client.println(S);
  Serial.println(S);
  delay(50);

  // Read all the lines of the reply from server and print them to Serial
  while(client.available())
    {
    String line = client.readStringUntil('\r');
    Serial.print(line);
    if (line=="OK")
      break;
    }
  
  Serial.println();
  Serial.println("closing connection");
  digitalWrite(BUILTIN_LED, HIGH);                                                         
  
  delay(100);
  ESP.deepSleep(30000000); // ,WAKE_RF_DISABLED);
}


String dsJson("{");

OneWire  ds(4); 

bool bReadDht=false;
void ReadDht()
{
  // Reading temperature or humidity takes about 250 milliseconds!
  // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)
  float h=NAN;
  float t=NAN;
  if (!dht.readAll(t,h))
    {
    Serial.println("Failed to read from DHT sensor!");
    return;
    }

  float hic = dht.computeHeatIndex(t, h, false);   
  if (dsJson.length()>1)
    dsJson+=String(",\"h\":");
  else  
    dsJson+=String("\"h\":");
  dsJson+=String(h, 2);  
  dsJson+=String(",\"t\":");
  dsJson+=String(t, 2);  
  dsJson+=String(",\"hic\":");
  dsJson+=String(hic, 2);  
}
 

void loop(void) 
{
  digitalWrite(BUILTIN_LED, LOW);                                                
  if (!bReadDht)
    {
    ReadDht();
    bReadDht=true;
    }
  
  byte i;
  byte present = 0;
  byte type_s;
  byte data[12];
  byte addr[8];
  float celsius, fahrenheit;
  
  if ( !ds.search(addr)) 
    {
    Serial.println("No more addresses.");
    Serial.println();
    ds.reset_search();
    dsJson+=F("}");
    Serial.print(F("Json:"));
    Serial.println(dsJson);
    WriteToServer(dsJson); 
    bReadDht=false;
    delay(250);
    dsJson="{";
    Serial.print("Free heap:");
    Serial.println(ESP.getFreeHeap(),DEC); 
    return;
    }
  String sAddr;
  
  Serial.print("ROM =");
  for ( i = 0; i < 8; i++) 
    {
    Serial.write(' ');
    Serial.print(addr[i], HEX);
    sAddr+=String(addr[i], HEX);
    }

  if (OneWire::crc8(addr, 7) != addr[7]) 
    {
    Serial.println("CRC is not valid!");
    return;
    }
  Serial.println();
 
  // the first ROM byte indicates which chip
  switch (addr[0]) {
    case 0x10:
      Serial.println("  Chip = DS18S20");  // or old DS1820
      type_s = 1;
      break;
    case 0x28:
      Serial.println("  Chip = DS18B20");
      type_s = 0;
      break;
    case 0x22:
      Serial.println("  Chip = DS1822");
      type_s = 0;
      break;
    default:
      Serial.println("Device is not a DS18x20 family device.");
      return;
  } 

  ds.reset();
  ds.select(addr);
  ds.write(0x44, 1);        // start conversion, with parasite power on at the end
  
  delay(1000);     // maybe 750ms is enough, maybe not
  // we might do a ds.depower() here, but the reset will take care of it.
  
  present = ds.reset();
  ds.select(addr);    
  ds.write(0xBE);         // Read Scratchpad

  Serial.print("  Data = ");
  Serial.print(present, HEX);
  Serial.print(" ");
  for ( i = 0; i < 9; i++) {           // we need 9 bytes
    data[i] = ds.read();
    Serial.print(data[i], HEX);
    Serial.print(" ");
  }
  Serial.print(" CRC=");
  Serial.print(OneWire::crc8(data, 8), HEX);
  Serial.println();

  // Convert the data to actual temperature
  // because the result is a 16 bit signed integer, it should
  // be stored to an "int16_t" type, which is always 16 bits
  // even when compiled on a 32 bit processor.
  int16_t raw = (data[1] << 8) | data[0];
  if (type_s) {
    raw = raw << 3; // 9 bit resolution default
    if (data[7] == 0x10) {
      // "count remain" gives full 12 bit resolution
      raw = (raw & 0xFFF0) + 12 - data[6];
    }
  } else {
    byte cfg = (data[4] & 0x60);
    // at lower res, the low bits are undefined, so let's zero them
    if (cfg == 0x00) raw = raw & ~7;  // 9 bit resolution, 93.75 ms
    else if (cfg == 0x20) raw = raw & ~3; // 10 bit res, 187.5 ms
    else if (cfg == 0x40) raw = raw & ~1; // 11 bit res, 375 ms
    //// default is 12 bit resolution, 750 ms conversion time
  }
  celsius = (float)raw / 16.0;
  fahrenheit = celsius * 1.8 + 32.0;
  Serial.print("  Temperature = ");
  Serial.print(celsius);
  Serial.print(" Celsius, ");
  Serial.print(fahrenheit);
  Serial.println(" Fahrenheit");
  
  if (dsJson.length()>1)
    dsJson+=String(",\"");
  else  
    dsJson+=String("\"");
  dsJson+=sAddr;
  dsJson+="\":";
  dsJson+=String(celsius, 2);  
}
